import json
import os
from aiohttp import web

from configparser import ConfigParser
from services.basicservice import BasicService
from singleton import Singleton
from dataclasses import dataclass

@dataclass
class ShiftData:
    backTotalSum:float = 0
    saleTotalSum:float = 0
    

class IKassa(BasicService, metaclass=Singleton):
    def __init__(self, servicePath: str):
        super().__init__(__file__)
        self.servicePath = servicePath
        self._loadAllCases()
        self.methods = {
            'get_token_state': "get_token_state_closed",
            'get_x_report': "get_x_report"
        }
        self.commandMap = {
            "open_shift": self.onOpenShift,
            "close_shift": self.onCloseShift,
            "get_token_state": self.onGetTokenState,
            "create_money_back": self.onCreateMoneyBack,
            "get_x_report": self.onGetXReport,
            #"rollback": self.onRollback
        }
        self.middleware = {
            "create_deposit": self.middlewareOnCreateDeposit,
            "create_withdraw": self.middlewareOnCreateWithdraw,
            "create_sale": self.middlewarenOnCreateSale,
            "create_money_back": self.middlewareOnCreateMoneyBack,
            "get_x_report": self.middlewareOnGetXReport
        }

        self.shiftData = ShiftData()

    def getRoutes(self) -> list[web.AbstractRouteDef]:
        return [web.post(self.servicePath + '/ik.service.app/version', handler=self.version),
                web.post(self.servicePath, handler=self.entryPoint)]

    async def version(self, request: web.Request) -> web.Response:
        self.log.info("Получение версии протокола ikassa")
        return self._createResponse('/ik.service.app/version')

    async def entryPoint(self, request: web.Request) -> web.Response:
        self.log.info("Получен запрос")
        request_data = await request.json()
        command = str(request_data['headers']['action']).lower()
        self.log.info(f"Метод '{command}'")
        return self.processCommand(command, request_data)

    def createResponse(self, command:str):
         if not command.startswith("/"):
             command = "/"+command
         return json.loads(self._createResponse(command).text)

    def processCommand(self, command: str, request_data: dict) -> web.Response:
        handler = self.commandMap.get(command)
        middleware = self.defaultMiddleware
        if command in self.middleware:
            middleware = self.middleware[command]

        if not handler:
            return middleware(request_data, self.createResponse(command))
        return middleware(request_data, handler(request_data))

    def defaultMiddleware(self, request_data: dict, any:json):
        return web.json_response(any)
    
    def onOpenShift(self, request_data: dict):
        self.shiftData = ShiftData()
        self.methods['get_token_state'] = "get_token_state"
        return self.createResponse("/open_shift")
    
    def onCloseShift(self, request_data: dict) -> web.Response:
        self.methods['get_token_state'] = "get_token_state_closed"
        return self.createResponse('/close_shift')

    def onGetTokenState(self, request_data: dict) -> web.Response:
        command = self.methods.get('get_token_state', 'get_token_state_closed')
        return self.createResponse(command)

    def onCreateMoneyBack(self, request_data: dict) -> web.Response:
        self.methods['get_x_report'] = "get_x_report_moneyback"
        return self.createResponse('/create_money_back')

    def onGetXReport(self, request_data: dict) -> web.Response:
        command = self.methods.get('get_x_report', 'get_x_report')
        return self.createResponse(command)
    
    def middlewarenOnCreateSale(self, request_data: dict, data:json):
        saleSum = 0.0
        for i in request_data['data']['sale']["payments"]:
            saleSum += float(i["value"])

        self.shiftData.saleTotalSum += saleSum 
        try:
            data['data']['payments'][0]['value'] = self.shiftData.saleTotalSum
            data['data']['totals']['sum'] = self.shiftData.saleTotalSum
        except:
            pass

        return data

    def middlewareOnCreateMoneyBack(self, request_data: dict, data:json):
        saleSum = 0.0
        for i in request_data['data']['money_back']["payments"]:
            saleSum += float(i["value"])

        self.shiftData.backTotalSum += saleSum 
        try:
            data['data']['payments'][0]['value'] = str(self.shiftData.backTotalSum)
            data['data']['totals']['sum'] = str(self.shiftData.backTotalSum)
        except:
            pass

        return web.json_response(data)
   
    def middlewareOnCreateDeposit(self, request_data: dict, data:json):
        data['data']['sum'] = request_data['data']['sum_cheque_data']['sum']
        return web.json_response(data)

    def middlewareOnCreateWithdraw(self, request_data: dict, data:json):
        data['data']['sum'] = request_data['data']['sum_cheque_data']['sum']
        return web.json_response(data)

    def middlewareOnGetXReport(self, request_data: dict, data:json):
        self.log.info(data)

        if "BYN" not in data['data']["currency_counters"]:
            return web.json_response(data) 
        
        data['data']['currency_counters']['BYN']['sales']['cash'] = str(self.shiftData.saleTotalSum)
        data['data']['currency_counters']['BYN']['sales']['card'] = str(self.shiftData.saleTotalSum)
        data['data']['currency_counters']['BYN']['sales']['sum'] = str(self.shiftData.saleTotalSum)

        data['data']['currency_counters']['BYN']['money_backs']['cash'] = str(self.shiftData.backTotalSum)
        data['data']['currency_counters']['BYN']['money_backs']['card'] = str(self.shiftData.backTotalSum)
        data['data']['currency_counters']['BYN']['money_backs']['sum'] = str(self.shiftData.backTotalSum)

        return web.json_response(data)
    

conf = ConfigParser()
conf.read(os.path.join(os.path.dirname(__file__), 'config.ini'))
servicePath = conf.get('ikassa', 'path') if conf.has_option('ikassa', 'path') else '/ikassa'
instance = IKassa(servicePath)